package service

import "XT_New/models"

func CreateVMOrgPatient(patients *models.VMOrgPatients) {
	writeDb.Model(&models.VMOrgPatients{}).Create(&patients)
}

func CreateVMZone(zone *models.VMDeviceZone) {
	writeDb.Model(&models.VMDeviceZone{}).Create(&zone)
}

func CreateVMGroup(group *models.VMDeviceGroup) {
	writeDb.Model(&models.VMDeviceGroup{}).Create(&group)
}

func CreateVMDeviceNumber(number *models.VMDeviceNumber) {
	writeDb.Model(&models.VMDeviceNumber{}).Create(&number)
}

func CreateVMSch(sch *models.VMSchedule) {
	writeDb.Model(&models.VMSchedule{}).Create(&sch)
}
